package br.com.rodrigosieja.brasilprev.controller;

import br.com.rodrigosieja.brasilprev.exception.CancelledPlanException;
import br.com.rodrigosieja.brasilprev.exception.InvalidAdditionalIntakeException;
import br.com.rodrigosieja.brasilprev.model.Plano;
import br.com.rodrigosieja.brasilprev.model.Produto;
import br.com.rodrigosieja.brasilprev.model.dto.AporteExtraDTO;
import br.com.rodrigosieja.brasilprev.model.dto.GenericResponseDTO;
import br.com.rodrigosieja.brasilprev.service.AporteExtraService;
import br.com.rodrigosieja.brasilprev.service.PlanoService;
import br.com.rodrigosieja.brasilprev.service.ProdutoService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.responses.ApiResponse;
import io.swagger.v3.oas.annotations.responses.ApiResponses;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

@RestController
@RequestMapping("/aporte-extra")
public class AporteExtraController {

    private final AporteExtraService aporteExtraService;
    private final PlanoService planoService;
    private final ProdutoService produtoService;

    @Autowired
    public AporteExtraController(AporteExtraService aporteExtraService,
                                 PlanoService planoService,
                                 ProdutoService produtoService) {
        this.aporteExtraService = aporteExtraService;
        this.planoService = planoService;
        this.produtoService = produtoService;
    }

    @PutMapping
    @Operation(summary = "Aporte Extra")
    @ApiResponses(value = {
            @ApiResponse(
                    responseCode = "200",
                    description = "Aporte Extra realizado"
            ),
            @ApiResponse(
                    responseCode = "400",
                    description = "Request inválido"
            )
    })
    public ResponseEntity<GenericResponseDTO> aporteExtra(final @RequestBody AporteExtraDTO aporteExtra) {
        if (!planoService.planoCancelado(aporteExtra.getIdPlano())) {
            Plano plano = planoService.busca(aporteExtra.getIdPlano());
            Produto produto = produtoService.busca(plano.getIdProduto());
            if (aporteExtraService.valorMinimoAporteProduto(aporteExtra.getValorAporte(), produto.getValorMinimoAporteExtra())) {
                GenericResponseDTO genericResponse = new GenericResponseDTO(aporteExtraService.aporteExtra(aporteExtra).getId());
                return ResponseEntity.ok(genericResponse);
            } else {
                throw new InvalidAdditionalIntakeException("Aporte extra inválido!");
            }
        } else {
            throw new CancelledPlanException("Plano cancelado! Não é possível realizar operações.");
        }
    }

}
