package br.com.rodrigosieja.brasilprev.controller;

import br.com.rodrigosieja.brasilprev.model.Cliente;
import br.com.rodrigosieja.brasilprev.model.dto.GenericResponseDTO;
import br.com.rodrigosieja.brasilprev.service.ClienteService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.responses.ApiResponse;
import io.swagger.v3.oas.annotations.responses.ApiResponses;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

@RestController
@RequestMapping("/cliente")
public class ClienteController {

    private final ClienteService clienteService;

    @Autowired
    public ClienteController(ClienteService clienteService) {
        this.clienteService = clienteService;
    }

    @PostMapping
    @Operation(summary = "Cadastro de cliente")
    @ApiResponses(value = {
            @ApiResponse(
                    responseCode = "201",
                    description = "Cliente inserido"
            ),
            @ApiResponse(
                    responseCode = "400",
                    description = "Request inválido"
            )
    })
    public ResponseEntity<GenericResponseDTO> insere(final @RequestBody Cliente cliente) {
        GenericResponseDTO genericResponse = new GenericResponseDTO(clienteService.insere(cliente).getId());
        return ResponseEntity.status(201).body(genericResponse);
    }

    @GetMapping("/buscar")
    @Operation(summary = "Lista de clientes")
    @ApiResponses(value = {
            @ApiResponse(
                    responseCode = "200",
                    description = "Clientes encontrados"
            ),
            @ApiResponse(
                    responseCode = "400",
                    description = "Request inválido"
            )
    })
    public ResponseEntity<List<Cliente>> lista() {
        return ResponseEntity.ok().body(clienteService.lista());
    }

}
