package br.com.rodrigosieja.brasilprev.controller;

import br.com.rodrigosieja.brasilprev.model.Produto;
import br.com.rodrigosieja.brasilprev.model.dto.GenericResponseDTO;
import br.com.rodrigosieja.brasilprev.service.ProdutoService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.responses.ApiResponse;
import io.swagger.v3.oas.annotations.responses.ApiResponses;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

@RestController
@RequestMapping("/produto")
public class ProdutoController {

    private final ProdutoService produtoService;

    @Autowired
    public ProdutoController(ProdutoService produtoService) {
        this.produtoService = produtoService;
    }

    @PostMapping
    @Operation(summary = "Cadastro de produto")
    @ApiResponses(value = {
            @ApiResponse(
                    responseCode = "201",
                    description = "Produto inserido"
            ),
            @ApiResponse(
                    responseCode = "400",
                    description = "Request inválido"
            )
    })
    public ResponseEntity<GenericResponseDTO> insere(final @RequestBody Produto produto) {
        GenericResponseDTO genericResponse = new GenericResponseDTO(produtoService.insere(produto).getId());
        return ResponseEntity.status(201).body(genericResponse);
    }

    @GetMapping("/buscar")
    @Operation(summary = "Lista de produtos")
    @ApiResponses(value = {
            @ApiResponse(
                    responseCode = "200",
                    description = "Produtos encontrados"
            ),
            @ApiResponse(
                    responseCode = "400",
                    description = "Request inválido"
            )
    })
    public ResponseEntity<List<Produto>> lista() {
        return ResponseEntity.ok().body(produtoService.lista());
    }

}
