package br.com.rodrigosieja.brasilprev.controller;

import br.com.rodrigosieja.brasilprev.exception.CancelledPlanException;
import br.com.rodrigosieja.brasilprev.exception.InvalidRedemptionException;
import br.com.rodrigosieja.brasilprev.model.Plano;
import br.com.rodrigosieja.brasilprev.model.Produto;
import br.com.rodrigosieja.brasilprev.model.dto.GenericResponseDTO;
import br.com.rodrigosieja.brasilprev.model.dto.ResgateDTO;
import br.com.rodrigosieja.brasilprev.service.PlanoService;
import br.com.rodrigosieja.brasilprev.service.ProdutoService;
import br.com.rodrigosieja.brasilprev.service.ResgateService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.responses.ApiResponse;
import io.swagger.v3.oas.annotations.responses.ApiResponses;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

@RestController
@RequestMapping("/resgate")
public class ResgateController {

    private final ResgateService resgateService;
    private final PlanoService planoService;
    private final ProdutoService produtoService;

    @Autowired
    public ResgateController(ResgateService resgateService,
                             PlanoService planoService,
                             ProdutoService produtoService) {
        this.resgateService = resgateService;
        this.planoService = planoService;
        this.produtoService = produtoService;
    }

    @PutMapping
    @Operation(summary = "Resgate")
    @ApiResponses(value = {
            @ApiResponse(
                    responseCode = "200",
                    description = "Resgate realizado"
            ),
            @ApiResponse(
                    responseCode = "400",
                    description = "Request inválido"
            )
    })
    public ResponseEntity<GenericResponseDTO> resgate(final @RequestBody ResgateDTO resgate) {
        if (!planoService.planoCancelado(resgate.getIdPlano())) {
            Plano plano = planoService.busca(resgate.getIdPlano());
            Produto produto = produtoService.busca(plano.getIdProduto());
            if (resgateService.resgateValido(resgate, plano, produto)) {
                GenericResponseDTO genericResponse = new GenericResponseDTO(resgateService.resgataAporte(resgate).getId());
                return ResponseEntity.ok(genericResponse);
            } else {
                throw new InvalidRedemptionException("Resgate inválido!");
            }
        } else {
            throw new CancelledPlanException("Plano cancelado! Não é possível realizar operações.");
        }
    }

}
