package br.com.rodrigosieja.brasilprev.model;

import br.com.rodrigosieja.brasilprev.enums.Genero;
import io.swagger.v3.oas.annotations.media.Schema;
import jakarta.persistence.EnumType;
import jakarta.persistence.Enumerated;
import jakarta.persistence.GenerationType;

import jakarta.persistence.Column;
import jakarta.persistence.Entity;
import jakarta.persistence.GeneratedValue;
import jakarta.persistence.Id;
import jakarta.persistence.Table;

import java.io.Serial;
import java.io.Serializable;
import java.math.BigDecimal;
import java.time.LocalDate;
import java.util.UUID;

@Schema(name = "Entidade Cliente")
@Entity
@Table(name = "cliente")
public class Cliente implements Serializable {

    @Serial
    private static final long serialVersionUID = -5706711842393569390L;

    @Id
    @GeneratedValue(strategy = GenerationType.UUID)
    @Schema(
            description = "ID",
            name = "id",
            type = "uuid",
            example = "7e468f2e-11b6-41d1-b9f9-ca46dd9f5d8e"
    )
    private UUID id;

    @Column(name = "cpf")
    @Schema(
            description = "CPF",
            name = "cpf",
            type = "string",
            example = "45645645600"
    )
    private String cpf;

    @Column(name = "nome")
    @Schema(
            description = "Nome",
            name = "nome",
            type = "string",
            example = "José da Silva"
    )
    private String nome;

    @Column(name = "email")
    @Schema(
            description = "E-mail",
            name = "email",
            type = "string",
            example = "jose@cliente.com"
    )
    private String email;

    @Column(name = "data_de_nascimento")
    @Schema(
            description = "Data de nascimento",
            name = "nome",
            type = "date",
            example = "2010-08-24T12:00:00.000Z"
    )
    private LocalDate dataDeNascimento;

    @Enumerated(EnumType.STRING)
    @Column(name = "genero")
    @Schema(
            description = "Gênero",
            name = "genero",
            type = "string",
            example = "Masculino"
    )
    private Genero genero;

    @Column(name = "renda_mensal")
    @Schema(
            description = "Renda mensal",
            name = "rendaMensal",
            type = "number",
            example = "2899.5"
    )
    private BigDecimal rendaMensal;

    public Cliente() {
    }

    public Cliente(String cpf, String nome, String email, LocalDate dataDeNascimento, Genero genero, BigDecimal rendaMensal) {
        this.cpf = cpf;
        this.nome = nome;
        this.email = email;
        this.dataDeNascimento = dataDeNascimento;
        this.genero = genero;
        this.rendaMensal = rendaMensal;
    }

    public UUID getId() {
        return id;
    }

    public void setId(UUID id) {
        this.id = id;
    }

    public String getCpf() {
        return cpf;
    }

    public void setCpf(String cpf) {
        this.cpf = cpf;
    }

    public String getNome() {
        return nome;
    }

    public void setNome(String nome) {
        this.nome = nome;
    }

    public String getEmail() {
        return email;
    }

    public void setEmail(String email) {
        this.email = email;
    }

    public LocalDate getDataDeNascimento() {
        return dataDeNascimento;
    }

    public void setDataDeNascimento(LocalDate dataDeNascimento) {
        this.dataDeNascimento = dataDeNascimento;
    }

    public Genero getGenero() {
        return genero;
    }

    public void setGenero(Genero genero) {
        this.genero = genero;
    }

    public BigDecimal getRendaMensal() {
        return rendaMensal;
    }

    public void setRendaMensal(BigDecimal rendaMensal) {
        this.rendaMensal = rendaMensal;
    }

    @Override
    public String toString() {
        return "Cliente{" +
                "id='" + id.toString() + '\'' +
                ", cpf='" + cpf + '\'' +
                ", nome='" + nome + '\'' +
                ", email='" + email + '\'' +
                ", dataDeNascimento=" + dataDeNascimento +
                ", genero='" + genero + '\'' +
                ", rendaMensal=" + rendaMensal.doubleValue() +
                '}';
    }

}
