package br.com.rodrigosieja.brasilprev.model;

import io.swagger.v3.oas.annotations.media.Schema;
import jakarta.persistence.GenerationType;
import org.hibernate.annotations.GenericGenerator;

import jakarta.persistence.Column;
import jakarta.persistence.Entity;
import jakarta.persistence.GeneratedValue;
import jakarta.persistence.Id;
import jakarta.persistence.Table;

import java.io.Serial;
import java.io.Serializable;
import java.math.BigDecimal;
import java.time.LocalDate;
import java.util.UUID;

@Schema(name = "Entidade Plano")
@Entity
@Table(name = "plano")
public class Plano implements Serializable {

    @Serial
    private static final long serialVersionUID = 5783382241187792910L;

    @Id
    @GeneratedValue(strategy = GenerationType.UUID)
    @Schema(
            description = "ID do plano",
            name = "id",
            type = "uuid",
            example = "414d3e4f-da78-40c5-bc9f-a35f923428f1"
    )
    private UUID id;

    @Column(name = "id_cliente")
    @Schema(
            description = "ID do cliente",
            name = "idCliente",
            type = "uuid",
            example = "18dfeb91-459a-4bc7-9cdd-d93b41f7bf62"
    )
    private UUID idCliente;

    @Column(name = "id_produto")
    @Schema(
            description = "ID do produto",
            name = "idProduto",
            type = "uuid",
            example = "30f6b23f-c93d-4cf9-8916-bcdb9fac83df"
    )
    private UUID idProduto;

    @Column(name = "aporte")
    @Schema(
            description = "Aporte",
            name = "aporte",
            type = "number",
            example = "2000.0"
    )
    private BigDecimal aporte;

    @Column(name = "data_da_contratacao")
    @Schema(
            description = "Data da contratação",
            name = "dataDaContratacao",
            type = "date",
            example = "2022-04-05T12:00:00.000Z"
    )
    private LocalDate dataDaContratacao;

    @Column(name = "idade_de_aposentadoria")
    @Schema(
            description = "Idade de Aposentadoria",
            name = "idadeDeAposentadoria",
            type = "number",
            example = "60"
    )
    private Integer idadeDeAposentadoria;

    public Plano() {
    }

    public Plano(UUID idCliente, UUID idProduto, BigDecimal aporte, LocalDate dataDaContratacao, Integer idadeDeAposentadoria) {
        this.idCliente = idCliente;
        this.idProduto = idProduto;
        this.aporte = aporte;
        this.dataDaContratacao = dataDaContratacao;
        this.idadeDeAposentadoria = idadeDeAposentadoria;
    }

    public UUID getId() {
        return id;
    }

    public void setId(UUID id) {
        this.id = id;
    }

    public UUID getIdCliente() {
        return idCliente;
    }

    public void setIdCliente(UUID idCliente) {
        this.idCliente = idCliente;
    }

    public UUID getIdProduto() {
        return idProduto;
    }

    public void setIdProduto(UUID idProduto) {
        this.idProduto = idProduto;
    }

    public BigDecimal getAporte() {
        return aporte;
    }

    public void setAporte(BigDecimal aporte) {
        this.aporte = aporte;
    }

    public LocalDate getDataDaContratacao() {
        return dataDaContratacao;
    }

    public void setDataDaContratacao(LocalDate dataDaContratacao) {
        this.dataDaContratacao = dataDaContratacao;
    }

    public Integer getIdadeDeAposentadoria() {
        return idadeDeAposentadoria;
    }

    public void setIdadeDeAposentadoria(Integer idadeDeAposentadoria) {
        this.idadeDeAposentadoria = idadeDeAposentadoria;
    }

    @Override
    public String toString() {
        return "Plano{" +
                "id=" + id.toString() +
                ", idCliente=" + idCliente.toString() +
                ", idProduto=" + idProduto.toString() +
                ", aporte=" + aporte.doubleValue() +
                ", dataDaContratacao=" + dataDaContratacao.toString() +
                ", idadeDeAposentadoria=" + idadeDeAposentadoria.doubleValue() +
                '}';
    }
}
