package br.com.rodrigosieja.brasilprev.model;

import io.swagger.v3.oas.annotations.media.Schema;
import jakarta.persistence.GenerationType;
import org.hibernate.annotations.GenericGenerator;

import jakarta.persistence.Column;
import jakarta.persistence.Entity;
import jakarta.persistence.GeneratedValue;
import jakarta.persistence.Id;
import jakarta.persistence.Table;

import java.io.Serial;
import java.io.Serializable;
import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.UUID;


@Schema(name = "Entidade Produto")
@Entity
@Table(name = "produto")
public class Produto implements Serializable {

    @Serial
    private static final long serialVersionUID = -5338985336640336822L;

    @Id
    @GeneratedValue(strategy = GenerationType.UUID)
    @Schema(
            description = "ID",
            name = "id",
            type = "uuid",
            example = "04aac243-be46-4a72-830f-3f3d72f6082e"
    )
    private UUID id;

    @Column(name = "nome")
    @Schema(
            description = "Nome",
            name = "nome",
            type = "string",
            example = "Brasilprev Longo Prazo"
    )
    private String nome;

    @Column(name = "susep")
    @Schema(
            description = "SUSEP",
            name = "susep",
            type = "string",
            example = "15414900840201817"
    )
    private String susep;

    @Column(name = "expiracao_de_venda")
    @Schema(
            description = "Expiração de venda",
            name = "expiracaoDeVenda",
            type = "date",
            example = "2021-01-01T12:00:00.000Z"
    )
    private LocalDateTime expiracaoDeVenda;

    @Column(name = "valor_minimo_aporte_inicial")
    @Schema(
            description = "Valor mínimo de aporte no momento da contratação",
            name = "valorMinimoAporteInicial",
            type = "number",
            example = "1000.0"
    )
    private BigDecimal valorMinimoAporteInicial;

    @Column(name = "valor_minimo_aporte_extra")
    @Schema(
            description = "Valor mínimo do aporte extra",
            name = "valorMinimoAporteExtra",
            type = "number",
            example = "100.0"
    )
    private BigDecimal valorMinimoAporteExtra;

    @Column(name = "idade_de_entrada")
    @Schema(
            description = "Idade mínima para comprar o produto",
            name = "idadeDeEntrada",
            type = "number",
            example = "10"
    )
    private Integer idadeDeEntrada;

    @Column(name = "idade_de_saida")
    @Schema(
            description = "Idade máxima para usufruir do benefício",
            name = "idadeDeSaida",
            type = "number",
            example = "60"
    )
    private Integer idadeDeSaida;

    @Column(name = "carencia_inicial_de_resgate")
    @Schema(
            description = "Em dias - carência para realizar o primeiro resgate",
            name = "carenciaInicialDeResgate",
            type = "number",
            example = "60"
    )
    private Integer carenciaInicialDeResgate;

    @Column(name = "carencia_entre_resgates")
    @Schema(
            description = "Em dias - carência para realizar outro resgate após um resgate realizado",
            name = "carenciaEntreResgates",
            type = "number",
            example = "30"
    )
    private Integer carenciaEntreResgates;

    public Produto() {
    }

    public Produto(String nome, String susep, LocalDateTime expiracaoDeVenda, BigDecimal valorMinimoAporteInicial, BigDecimal valorMinimoAporteExtra, Integer idadeDeEntrada, Integer idadeDeSaida, Integer carenciaInicialDeResgate, Integer carenciaEntreResgates) {
        this.nome = nome;
        this.susep = susep;
        this.expiracaoDeVenda = expiracaoDeVenda;
        this.valorMinimoAporteInicial = valorMinimoAporteInicial;
        this.valorMinimoAporteExtra = valorMinimoAporteExtra;
        this.idadeDeEntrada = idadeDeEntrada;
        this.idadeDeSaida = idadeDeSaida;
        this.carenciaInicialDeResgate = carenciaInicialDeResgate;
        this.carenciaEntreResgates = carenciaEntreResgates;
    }

    public UUID getId() {
        return id;
    }

    public void setId(UUID id) {
        this.id = id;
    }

    public String getNome() {
        return nome;
    }

    public void setNome(String nome) {
        this.nome = nome;
    }

    public String getSusep() {
        return susep;
    }

    public void setSusep(String susep) {
        this.susep = susep;
    }

    public LocalDateTime getExpiracaoDeVenda() {
        return expiracaoDeVenda;
    }

    public void setExpiracaoDeVenda(LocalDateTime expiracaoDeVenda) {
        this.expiracaoDeVenda = expiracaoDeVenda;
    }

    public BigDecimal getValorMinimoAporteInicial() {
        return valorMinimoAporteInicial;
    }

    public void setValorMinimoAporteInicial(BigDecimal valorMinimoAporteInicial) {
        this.valorMinimoAporteInicial = valorMinimoAporteInicial;
    }

    public BigDecimal getValorMinimoAporteExtra() {
        return valorMinimoAporteExtra;
    }

    public void setValorMinimoAporteExtra(BigDecimal valorMinimoAporteExtra) {
        this.valorMinimoAporteExtra = valorMinimoAporteExtra;
    }

    public Integer getIdadeDeEntrada() {
        return idadeDeEntrada;
    }

    public void setIdadeDeEntrada(Integer idadeDeEntrada) {
        this.idadeDeEntrada = idadeDeEntrada;
    }

    public Integer getIdadeDeSaida() {
        return idadeDeSaida;
    }

    public void setIdadeDeSaida(Integer idadeDeSaida) {
        this.idadeDeSaida = idadeDeSaida;
    }

    public Integer getCarenciaInicialDeResgate() {
        return carenciaInicialDeResgate;
    }

    public void setCarenciaInicialDeResgate(Integer carenciaInicialDeResgate) {
        this.carenciaInicialDeResgate = carenciaInicialDeResgate;
    }

    public Integer getCarenciaEntreResgates() {
        return carenciaEntreResgates;
    }

    public void setCarenciaEntreResgates(Integer carenciaEntreResgates) {
        this.carenciaEntreResgates = carenciaEntreResgates;
    }

    @Override
    public String toString() {
        return "Produto{" +
                "id=" + id.toString() +
                ", nome='" + nome + '\'' +
                ", susep='" + susep + '\'' +
                ", expiracaoDeVenda=" + expiracaoDeVenda.toString() +
                ", valorMinimoAporteInicial=" + valorMinimoAporteInicial.doubleValue() +
                ", valorMinimoAporteExtra=" + valorMinimoAporteExtra.doubleValue() +
                ", idadeDeEntrada=" + idadeDeEntrada +
                ", idadeDeSaida=" + idadeDeSaida +
                ", carenciaInicialDeResgate=" + carenciaInicialDeResgate +
                ", carenciaEntreResgates=" + carenciaEntreResgates +
                '}';
    }
}
