package br.com.rodrigosieja.brasilprev.service;

import br.com.rodrigosieja.brasilprev.exception.PlanNotFoundException;
import br.com.rodrigosieja.brasilprev.model.Plano;
import br.com.rodrigosieja.brasilprev.repository.PlanoRepository;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.List;
import java.util.Optional;
import java.util.UUID;

@Service
public class PlanoService {

    private final PlanoRepository planoRepository;

    public PlanoService(PlanoRepository planoRepository) {
        this.planoRepository = planoRepository;
    }

    public Plano insere(final Plano plano) {
        return planoRepository.save(plano);
    }

    public Plano busca(final UUID id) {
        Optional<Plano> planoEncontrado = planoRepository.findById(id);
        return planoEncontrado
                .orElseThrow(() -> new PlanNotFoundException("Plano não encontrado!"));
    }

    public boolean prazoVendaExpirado(final LocalDateTime prazoVenda) {
        return LocalDateTime.now().isEqual(prazoVenda) || LocalDateTime.now().isBefore(prazoVenda);
    }

    public boolean aporteInicialValido(final BigDecimal valorAporte,
                                       final BigDecimal valorMinimoAporteInicial) {
        return valorAporte.doubleValue() >= valorMinimoAporteInicial.doubleValue();
    }

    public boolean idadeValida(final Integer idade,
                               final Integer idadeDeEntrada,
                               final Integer idadeDeSaida) {
        return idade.doubleValue() >= idadeDeEntrada.doubleValue()
                && idade.doubleValue() <= idadeDeSaida.doubleValue();
    }

    public boolean planoCancelado(final UUID idPlano) {
        return busca(idPlano).getAporte().doubleValue() == 0d;
    }

    public List<Plano> lista() {
        return planoRepository.findAll();
    }
}
