package br.com.rodrigosieja.brasilprev.service;

import br.com.rodrigosieja.brasilprev.model.Plano;
import br.com.rodrigosieja.brasilprev.model.Produto;
import br.com.rodrigosieja.brasilprev.model.dto.ResgateDTO;
import br.com.rodrigosieja.brasilprev.repository.PlanoRepository;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.time.LocalDate;
import java.time.Period;
import java.util.Optional;

@Service
public class ResgateService {

    private final PlanoRepository planoRepository;

    public ResgateService(PlanoRepository planoRepository) { this.planoRepository = planoRepository; }

    public Plano resgataAporte(final ResgateDTO resgate) {
        Optional<Plano> planoEncontrado = planoRepository.findById(resgate.getIdPlano());
        if (planoEncontrado.isPresent()) {
            Plano plano = planoEncontrado.get();
            BigDecimal aporteAnterior = plano.getAporte();
            BigDecimal aporteNovo = aporteAnterior.subtract(resgate.getValorResgate());
            plano.setAporte(aporteNovo);
            return planoRepository.save(plano);
        }

        return null;
    }

    public boolean resgateValido(final ResgateDTO resgate,
                                 final Plano plano,
                                 final Produto produto) {
        LocalDate agora = LocalDate.now();
        LocalDate dataContratacaoPlano = plano.getDataDaContratacao();
        int diasDesdeContratacao = Period.between(dataContratacaoPlano, agora).getDays();

        boolean carenciaValida = diasDesdeContratacao < produto.getCarenciaInicialDeResgate();
        boolean aporteValido = resgate.getValorResgate().doubleValue() <= plano.getAporte().doubleValue();

        return carenciaValida && aporteValido;
    }

}
