package br.com.rodrigosieja.brasilprev.controller;

import br.com.rodrigosieja.brasilprev.enums.Genero;
import br.com.rodrigosieja.brasilprev.model.Cliente;
import br.com.rodrigosieja.brasilprev.model.dto.GenericResponseDTO;
import br.com.rodrigosieja.brasilprev.service.ClienteService;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.junit.jupiter.MockitoExtension;
import org.springframework.http.ResponseEntity;
import org.springframework.test.context.TestPropertySource;

import java.math.BigDecimal;
import java.time.LocalDate;
import java.util.List;
import java.util.UUID;

import static org.assertj.core.api.Assertions.assertThat;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.Mockito.when;

@TestPropertySource("classpath:application-test.yml")
@ExtendWith(MockitoExtension.class)
public class ClienteControllerTest {

    @Mock
    private ClienteService clienteService;

    @InjectMocks
    private ClienteController clienteController;

    @Mock
    private Cliente clienteRequest;

    @Mock
    private Cliente clienteResponse;

    private UUID idCliente;

    @BeforeEach
    void setup() {
        clienteRequest = new Cliente(
                "45645645600",
                "José da Silva",
                "jose@cliente.com",
                LocalDate.of(2000, 1, 1),
                Genero.Masculino,
                BigDecimal.valueOf(2899.50)
        );

        idCliente = UUID.randomUUID();

        clienteResponse = new Cliente(
                "45645645600",
                "José da Silva",
                "jose@cliente.com",
                LocalDate.of(2000, 1, 1),
                Genero.Masculino,
                BigDecimal.valueOf(2899.50)
        );

        clienteResponse.setId(idCliente);
    }

    @Test
    public void insereTest() {
        when(clienteService.insere(any())).thenReturn(clienteResponse);

        ResponseEntity<GenericResponseDTO> response = clienteController.insere(clienteRequest);
        assertThat(response.getStatusCode().value()).isEqualTo(201);
    }

    @Test
    public void listaTest() {
        ResponseEntity<List<Cliente>> response = clienteController.lista();
        assertThat(response.getStatusCode().value()).isEqualTo(200);
    }

}
