package br.com.rodrigosieja.brasilprev.controller;

import br.com.rodrigosieja.brasilprev.model.Produto;
import br.com.rodrigosieja.brasilprev.model.dto.GenericResponseDTO;
import br.com.rodrigosieja.brasilprev.service.ProdutoService;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.junit.jupiter.MockitoExtension;
import org.springframework.http.ResponseEntity;
import org.springframework.test.context.TestPropertySource;

import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.List;
import java.util.UUID;

import static org.assertj.core.api.Assertions.assertThat;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.Mockito.when;

@TestPropertySource("classpath:application-test.yml")
@ExtendWith(MockitoExtension.class)
public class ProdutoControllerTest {

    @Mock
    private ProdutoService produtoService;

    @InjectMocks
    private ProdutoController produtoController;

    @Mock
    private Produto produtoRequest;

    @Mock
    private Produto produtoResponse;

    private UUID idProduto;

    @BeforeEach
    void setup() {
        produtoRequest = new Produto(
                "Brasilprev Longo Prazo",
                "15414900840201817",
                LocalDateTime.of(2021, 1, 1, 12, 0, 0),
                BigDecimal.valueOf(1000.0),
                BigDecimal.valueOf(100.0),
                18,
                60,
                60,
                30
        );

        idProduto = UUID.randomUUID();

        produtoResponse = new Produto(
                "Brasilprev Longo Prazo",
                "15414900840201817",
                LocalDateTime.of(2021, 1, 1, 12, 0, 0),
                BigDecimal.valueOf(1000.0),
                BigDecimal.valueOf(100.0),
                18,
                60,
                60,
                30
        );

        produtoResponse.setId(idProduto);
    }

    @Test
    public void insereTest() {
        when(produtoService.insere(any())).thenReturn(produtoResponse);

        ResponseEntity<GenericResponseDTO> response = produtoController.insere(produtoRequest);
        assertThat(response.getStatusCode().value()).isEqualTo(201);
    }

    @Test
    public void listaTest() {
        ResponseEntity<List<Produto>> response = produtoController.lista();
        assertThat(response.getStatusCode().value()).isEqualTo(200);
    }

}
