package br.com.rodrigosieja.brasilprev.service;

import br.com.rodrigosieja.brasilprev.enums.Genero;
import br.com.rodrigosieja.brasilprev.model.Cliente;
import br.com.rodrigosieja.brasilprev.repository.ClienteRepository;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.junit.jupiter.MockitoExtension;
import org.springframework.test.context.TestPropertySource;

import java.math.BigDecimal;
import java.time.LocalDate;
import java.util.List;
import java.util.Optional;
import java.util.UUID;

import static org.junit.jupiter.api.Assertions.assertFalse;
import static org.junit.jupiter.api.Assertions.assertNotNull;
import static org.junit.jupiter.api.Assertions.assertTrue;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.Mockito.when;

@TestPropertySource("classpath:application-test.yml")
@ExtendWith(MockitoExtension.class)
class ClienteServiceTest {

    @Mock
    private ClienteRepository repository;

    @InjectMocks
    private ClienteService service;

    @Mock
    private Cliente clienteRequest;

    @Mock
    private Cliente clienteResponse;

    private UUID idCliente;

    @BeforeEach
    void setup() {
        clienteRequest = new Cliente(
                "45645645600",
                "José da Silva",
                "jose@cliente.com",
                LocalDate.of(2000, 1, 1),
                Genero.Masculino,
                BigDecimal.valueOf(2899.50)
        );

        idCliente = UUID.randomUUID();

        clienteResponse = new Cliente(
                "45645645600",
                "José da Silva",
                "jose@cliente.com",
                LocalDate.of(2000, 1, 1),
                Genero.Masculino,
                BigDecimal.valueOf(2899.50)
        );

        clienteResponse.setId(idCliente);
    }

    @Test
    void insereTest() {
        when(repository.save(any())).thenReturn(clienteResponse);

        Cliente clienteInserido = service.insere(clienteRequest);

        assertNotNull(clienteInserido.getId());
    }

    @Test
    void buscaTest() {
        when(repository.findById(any())).thenReturn(Optional.of(clienteResponse));

        Cliente clienteEncontrado = service.busca(idCliente);

        assertNotNull(clienteEncontrado.getId());
    }

    @Test
    void listaTest() {
        when(repository.findAll()).thenReturn(List.of(clienteResponse));

        List<Cliente> clientes = service.lista();

        assertFalse(clientes.isEmpty());
    }

}
