package br.com.rodrigosieja.brasilprev.service;

import br.com.rodrigosieja.brasilprev.enums.Genero;
import br.com.rodrigosieja.brasilprev.model.Cliente;
import br.com.rodrigosieja.brasilprev.model.Plano;
import br.com.rodrigosieja.brasilprev.model.Produto;
import br.com.rodrigosieja.brasilprev.repository.PlanoRepository;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.junit.jupiter.MockitoExtension;
import org.springframework.test.context.TestPropertySource;

import java.math.BigDecimal;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.List;
import java.util.Optional;
import java.util.UUID;

import static org.junit.jupiter.api.Assertions.assertFalse;
import static org.junit.jupiter.api.Assertions.assertNotNull;
import static org.junit.jupiter.api.Assertions.assertTrue;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.Mockito.when;

@TestPropertySource("classpath:application-test.yml")
@ExtendWith(MockitoExtension.class)
class PlanoServiceTest {

    @Mock
    private PlanoRepository repository;

    @InjectMocks
    private PlanoService service;

    @Mock
    private Cliente cliente;

    @Mock
    private Produto produto;

    private UUID idCliente;

    private UUID idProduto;

    @Mock
    private Plano planoRequest;

    @Mock
    private Plano planoResponse;

    private UUID idPlano;

    @BeforeEach
    void setup() {
        idCliente = UUID.randomUUID();
        idProduto = UUID.randomUUID();

        idPlano = UUID.randomUUID();

        cliente = new Cliente(
                "45645645600",
                "José da Silva",
                "jose@cliente.com",
                LocalDate.of(2000, 1, 1),
                Genero.Masculino,
                BigDecimal.valueOf(2899.50)
        );
        cliente.setId(idCliente);

        produto = new Produto(
                "Brasilprev Longo Prazo",
                "15414900840201817",
                LocalDateTime.of(2023, 12, 4, 12, 0, 0),
                BigDecimal.valueOf(1000.0),
                BigDecimal.valueOf(100.0),
                18,
                60,
                60,
                30
        );
        produto.setId(idProduto);

        planoRequest = new Plano(
                idCliente,
                idProduto,
                BigDecimal.valueOf(1000.0),
                LocalDate.of(2024, 12, 4),
                60
        );

        planoResponse = new Plano(
                idCliente,
                idProduto,
                BigDecimal.valueOf(1000.0),
                LocalDate.of(2024, 12, 4),
                60
        );
        planoResponse.setId(idPlano);
    }

    @Test
    void insereTest() {
        when(repository.save(any())).thenReturn(planoResponse);

        Plano planoInserido = service.insere(planoRequest);

        assertNotNull(planoInserido.getId());
    }

    @Test
    void buscaTest() {
        when(repository.findById(any())).thenReturn(Optional.of(planoResponse));

        Plano planoEncontrado = service.busca(idPlano);

        assertNotNull(planoEncontrado.getId());
    }

    @Test
    void listaTest() {
        when(repository.findAll()).thenReturn(List.of(planoResponse));

        List<Plano> planos = service.lista();

        assertFalse(planos.isEmpty());
    }

    @Test
    void prazoVendaExpiradoTest() {
        boolean retorno = service.prazoVendaExpirado(LocalDateTime.now().plusDays(1L));
        assertTrue(retorno);
    }

    @Test
    void aporteInicialValidoTest() {
        boolean retorno = service.aporteInicialValido(BigDecimal.valueOf(1100.0), BigDecimal.valueOf(1000.0));
        assertTrue(retorno);
    }

    @Test
    void idadeValidaTest() {
        boolean retorno = service.idadeValida(25, 18, 60);
        assertTrue(retorno);
    }

    @Test
    void planoCanceladoTest() {
        when(repository.findById(any())).thenReturn(Optional.of(planoResponse));

        boolean retorno = service.planoCancelado(idPlano);
        assertFalse(retorno);
    }

}
