package br.com.rodrigosieja.brasilprev.service;

import br.com.rodrigosieja.brasilprev.model.Produto;
import br.com.rodrigosieja.brasilprev.repository.ProdutoRepository;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.junit.jupiter.MockitoExtension;
import org.springframework.test.context.TestPropertySource;

import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.List;
import java.util.Optional;
import java.util.UUID;

import static org.junit.jupiter.api.Assertions.assertFalse;
import static org.junit.jupiter.api.Assertions.assertNotNull;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.Mockito.when;

@TestPropertySource("classpath:application-test.yml")
@ExtendWith(MockitoExtension.class)
class ProdutoServiceTest {

    @Mock
    private ProdutoRepository repository;

    @InjectMocks
    private ProdutoService service;

    @Mock
    private Produto produtoRequest;

    @Mock
    private Produto produtoResponse;

    private UUID idProduto;

    @BeforeEach
    void setup() {
        produtoRequest = new Produto(
                "Brasilprev Longo Prazo",
                "15414900840201817",
                LocalDateTime.of(2024, 12, 4, 12, 0, 0),
                BigDecimal.valueOf(1000.0),
                BigDecimal.valueOf(100.0),
                18,
                60,
                60,
                30
        );

        idProduto = UUID.randomUUID();

        produtoResponse = new Produto(
                "Brasilprev Longo Prazo",
                "15414900840201817",
                LocalDateTime.of(2024, 12, 4, 12, 0, 0),
                BigDecimal.valueOf(1000.0),
                BigDecimal.valueOf(100.0),
                18,
                60,
                60,
                30
        );

        produtoResponse.setId(idProduto);
    }

    @Test
    void insereTest() {
        when(repository.save(any())).thenReturn(produtoResponse);

        Produto produtoInserido = service.insere(produtoRequest);

        assertNotNull(produtoInserido.getId());
    }

    @Test
    void buscaTest() {
        when(repository.findById(any())).thenReturn(Optional.of(produtoResponse));

        Produto produtoEncontrado = service.busca(idProduto);

        assertNotNull(produtoEncontrado.getId());
    }

    @Test
    void listaTest() {
        when(repository.findAll()).thenReturn(List.of(produtoResponse));

        List<Produto> produtos = service.lista();

        assertFalse(produtos.isEmpty());
    }

}
