package br.com.rodrigosieja.brasilprev.service;

import br.com.rodrigosieja.brasilprev.model.Plano;
import br.com.rodrigosieja.brasilprev.model.Produto;
import br.com.rodrigosieja.brasilprev.model.dto.ResgateDTO;
import br.com.rodrigosieja.brasilprev.repository.PlanoRepository;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.junit.jupiter.MockitoExtension;
import org.springframework.test.context.TestPropertySource;

import java.math.BigDecimal;
import java.time.LocalDate;
import java.util.Optional;
import java.util.UUID;

import static org.junit.jupiter.api.Assertions.assertNotNull;
import static org.junit.jupiter.api.Assertions.assertNull;
import static org.junit.jupiter.api.Assertions.assertTrue;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.Mockito.when;

@TestPropertySource("classpath:application-test.yml")
@ExtendWith(MockitoExtension.class)
class ResgateServiceTest {

    @Mock
    private PlanoRepository planoRepository;

    @InjectMocks
    private ResgateService service;

    @Mock
    private ResgateDTO resgate;

    private UUID idCliente;

    private UUID idProduto;

    @Mock
    private Plano planoRequest;

    @Mock
    private Plano planoResponse;

    private UUID idPlano;

    @BeforeEach
    void setup() {
        idCliente = UUID.randomUUID();
        idProduto = UUID.randomUUID();

        idPlano = UUID.randomUUID();

        resgate = new ResgateDTO(
                idPlano,
                BigDecimal.valueOf(100.0)
        );

        planoResponse = new Plano(
                idCliente,
                idProduto,
                BigDecimal.valueOf(900.0),
                LocalDate.of(2024, 12, 4),
                60
        );
        planoResponse.setId(idPlano);
    }

    @Test
    void resgataAporteTest() {
        when(planoRepository.findById(any())).thenReturn(Optional.of(planoResponse));
        when(planoRepository.save(any())).thenReturn(planoResponse);

        Plano plano = service.resgataAporte(resgate);

        assertNotNull(plano.getId());
    }

    @Test
    void resgataAportePlanoNaoEncontradoTest() {
        when(planoRepository.findById(any())).thenReturn(Optional.empty());

        Plano plano = service.resgataAporte(resgate);

        assertNull(plano);
    }

    @Test
    void resgateValidoTest() {
        Produto produto = new Produto();
        produto.setCarenciaInicialDeResgate(30);

        boolean retorno = service.resgateValido(resgate, planoResponse, produto);

        assertTrue(retorno);
    }


}
